// @eslint-disable
import React, {
  Suspense,
  useEffect,
  useRef,
  useState,
} from 'react';

import {
  Anchor,
  Popover,
  Radio,
  Spin,
} from 'antd';
import classNames from 'classnames';
import Cookies from 'js-cookie';
import _ from 'lodash';
import memoized from 'nano-memoize';
import KeepAlive from 'react-activation';
import { connect } from 'react-redux';
import { matchRoutes } from 'react-router-dom';

import RightContent from '@/components/GlobalHeader/SimpleRiightContent';
import defaultSetting from '@wisdom-utils/components/lib/AppLayout/layouts/defaultSettings';
// import SettingDrawer from '@wisdom-utils/components/lib/layout/components/SettingDrawer';
import { actionCreators } from '@/containers/App/store';
import Icon, { LoadingOutlined } from '@ant-design/icons';
import BasicLayout from '@wisdom-utils/components/lib/layout/BasicLayout';
import {
  useHistory,
  useLocation,
} from '@wisdom-utils/runtime';
import {
  helpers,
  store,
} from '@wisdom-utils/utils';

import { waterMark } from '../utils/mark';
import layoutStyles from './BasicLayout.less';
import SecurityLayout from './SecurityLayout';
import Site from './Site';

const { params } = helpers;
const { getParamsV1 } = params;

// const ArcgisMap = React.lazy(() => import('../pages/map/arcgis'));
// const AMap = React.lazy(() => import('../pages/map/amap'));
const CesiumMap = React.lazy(() => import('../pages/map/cesiumMap'));

const waitTime = (time= 100) => {
  return new Promise((resolve) => {
    setTimeout(() => {
      resolve(true);
    }, time);
  });
};
const antIcon = <LoadingOutlined style={{ fontSize: 12 }} spin />;
const arrowSvg = ({ fillColor = '#fff' }) => (
  <svg
    t="1543324489942"
    className="icon"
    viewBox="0 0 1024 1024"
    version="1.1"
    xmlns="http://www.w3.org/2000/svg"
    width="16"
    height="16"
  >
    <path
      d="M511.700683 639.423111 191.917496 319.596945 319.830771 319.596945 511.700683 511.715521 703.570595 319.596945 831.48387 319.596945Z"
      p-id="8626"
      // fill={fillColor}
    />
  </svg>
);
const ArrowIcon = props => <Icon component={arrowSvg} {...props} />;
const optionsWith = [{ label: '按站点', value: 'site' }, { label: '按城市', value: 'city' }];
const HOT = ['HOT', '县', '市', 'New'];
const StationsItem = (item, action, onChangeVisible, actionRef, setMenuLoading) => {
  const changeGroup = (event, data) => {
    action.changeGroup && action.changeGroup(event, data, onChangeVisible, actionRef);
    setMenuLoading && setMenuLoading(true);
  };
  return (
    <li key={item.groupID} style={{ marginRight: `${item.style.marginRight}px` }}>
      <a
        className={classNames(layoutStyles.city_select, layoutStyles.showTip)}
        title={item.city}
        index={item.promoteIndex}
        style={{ fontSize: '13px' }}
        onClick={event => changeGroup(event, item)}
      >
        {item.groupName}
        {item.promoteTip && HOT.includes(item.promoteTip) && <div title={`${item.promoteTip}`}>{item.promoteTip}</div>}
      </a>
    </li>
  );
};

const OnlyFocusStations = props => {
  const data = props.data.stations;
  return (
    <Spin spinning={props.loading} tip="加载中" size="small">
      <div className={layoutStyles.focusStations} style={{border: 'none'}}>
        <ul>{Array.isArray(data) ? data.map(item => StationsItem(item, props.action, props.setVisible, props.actionRef, props.setMenuLoading)) : null}</ul>
      </div>
    </Spin>
  )
}

const AllStations = props => {
  const data = props.data.stations;
  const [targetOffset, setTargetOffset] = useState(undefined);
  const cityPane = useRef(null);
  const cityContent = useRef(null);
  const [defaultTab, setDefaultTab] = useState('site');
  useEffect(() => {
    setTargetOffset(cityPane.current.clientHeight / 2);
  }, []);

  const handleClick = (event, link) => {
    event.preventDefault();
  };

  const handleTabChange = event => {
    setDefaultTab(event.target.value);
  };
  return (
    <>
      <div className={layoutStyles.focusStations}>
        <ul>{Array.isArray(data) ? data.map(item => StationsItem(item, props.action, props.setVisible, props.actionRef, props.setMenuLoading)) : null}</ul>
      </div>
      <Radio.Group
        options={optionsWith}
        optionType="button"
        buttonStyle="solid"
        size="small"
        value={defaultTab}
        onChange={handleTabChange}
        style={{ marginTop: '6px' }}
      />
      <div style={{ maxWidth: '520px', position: 'relative' }}>
        {defaultTab === 'site' ? (
          <div className={classNames(layoutStyles.city_pane, layoutStyles.station_container)} ref={cityPane}>
            <Anchor
              affix={false}
              onClick={handleClick}
              targetOffset={targetOffset}
              getContainer={() => cityContent.current}
            >
              <ul className={layoutStyles.py}>
                {props.data && props.data.siteCityList && props.data.siteCityList.letters}
              </ul>
              <div className={layoutStyles.cityContent} style={{ height: '335px' }} ref={cityContent}>
                {props.data && props.data.siteCityList && props.data.siteCityList.content}
              </div>
            </Anchor>
          </div>
        ) : (
          <div className="city_pane city_container">{props.data.citySelector}</div>
        )}
        <Spin spinning={props.loading} tip="加载中" />
      </div>
    </>
  );
};

const Stations = props => {
  const isAll = 
    props.data &&
    props.data.siteCityList &&
    ((props.data.siteCityList.letters && props.data.siteCityList.letters.length) ||
      (props.data.siteCityList.content && props.data.siteCityList.content.length));
  return isAll ? <AllStations {...props}/> : <OnlyFocusStations {...props}/>;
}

const renderSite = ({ data, config, loading, setLoading, action, actionRef, setMenuLoading }) => {
  const [visible, setVisible] = useState(false);
  let loaded = !!((data && !data.stations) || (Array.isArray(data.weathers) && data.weathers.length === 0));
  if (config && config.userInfo && config.userInfo.site === '') {
    loaded = false;
  }
  if(!data.allStation || !_.isArray(data.allStation) || data.allStation.length <= 1)
    return <></>;
  return (
    <>
      <Spin
        indicator={antIcon}
        spinning={loaded}
        size="small"
        style={{ marginLeft: '10px' }}
        tip=""
        wrapperClassName={layoutStyles.spinLoadding}
      >
        <>
          <Popover
            placement="bottomLeft"
            trigger="click"
            visible={visible}
            content={
              <Stations data={data} loading={loading} setLoading={setLoading} setVisible={setVisible} action={action} actionRef={actionRef} setMenuLoading={setMenuLoading} />
            }
            arrowPointAtCenter
            overlayClassName={classNames(layoutStyles.stationsWrapper, layoutStyles.stationsTop)}
            onVisibleChange={visible => setVisible(visible)}
          >
            {Array.isArray(data.stations) ? (
              <div className={layoutStyles.toggleSite}>
                <img src={require('../assets/basic/site.png')} className={layoutStyles.site} />
                <span className={layoutStyles.name}>{data.currentStationName}</span>
                <ArrowIcon
                  className={layoutStyles.arrow}
                  // fillColor="#fff"
                  style={{
                    transform: !visible ? `rotate(0deg)` : `rotate(180deg)`,
                  }}
                />
              </div>
            ) : null}
          </Popover>
        </>
      </Spin>
    </>
  );
};

// import defaultProps from '../defaultProps';
const pickRoutes = memoized((routes, pathname, locale) => {

    const matches = matchRoutes(routes, { pathname });
    const routeConfig = matches ? matches[matches.length - 1].route : null;

    return {
      locale, // just for cache
      routeConfig,
      // matchPath: matches ? matches.map(match => _.replace(match.route.path,'/*','')).join('/') : null // 解决下微端/*路径的问题
      matchPath: routeConfig ? _.replace(routeConfig.key, "/*", "") : '',
    };
});

const transformFloatMenu = (routes, homepage) => {
  const route = routes.find(route => route.path === `/${homepage}`);
  console.log("route", route);
  return routes.concat(route);
}

const Layout = (props) => {
  const [cityData, setCityData] = useState({});
  const [siteLoading, setSiteLoading] = useState(false);
  const [siteAction, setSiteAction] = useState(() => new Site(props, setSiteLoading));
  const [menuLoading, setMenuLoading] = useState(true);
  // const [settings, setSetting] = useState({});
  // const [routeConfig, setRouteConfig] = useState(null);
  // const [matchPath, setMatchPath] = useState('');
  const actionRef = useRef();
  const history = useHistory();
  const location = useLocation();
  const { routeConfig = {}, matchPath } = pickRoutes(props.flatMenu, location.pathname);
  // useEffect(() => {

  //   setRouteConfig(routeConfig);
  //   setMatchPath(match)
  // }, [location.pathname])



  useEffect(() => {
    siteAction.setGlobalConfig(props.global);
    const tk = Cookies.get('token') || props.global.token;
    const isLogin = tk !== null && tk !== 'undefined' && tk !== (void 0);
    if (!isLogin) {
      let client = props.global && props.global.hasOwnProperty('client') ? props.global.get('client') : null;
      client = client !== 'undefined' && !_.isNull(client) && !_.isUndefined(client) ? client : 'city';
      let generateType = props.global && props.global.hasOwnProperty('get') ? props.global.get('generateType') : null;
      generateType =
        !_.isNull(generateType) && !_.isUndefined(generateType) && generateType !== 'undefined'
          ? `&generateType=${generateType}`
          : '';
      history.replace(`/user/login?client=${client}${generateType}`);
      props.logout();
    }
  }, [history, props, props.global, siteAction]);

  useEffect(() => {
    if (
      props.global &&
      props.global.userInfo &&
      props.global.userInfo.token !== null &&
      props.global.userInfo.loginName &&
      Object.keys(cityData).length === 0
    ) {
      siteAction.getCityStationsForUser().then(res => {
        setCityData(res);
      });
    }
  }, [cityData, props.global, props.global.userInfo, siteAction]);


  useEffect(() => {
    window.share.event.on('updateSite', res => {
      setCityData(res);
    });
    return () => {
      window.share.event.removeAllListeners('updateSite');
    };
  }, []);

  useEffect(() => {
    const needMark = () =>
      (props.global.isHaveMark || window.location.origin.replace(/^(http|https):\/\//, '') === 'panda-water.cn') &&
      document.querySelectorAll('.pandawatermark_mask_div').length <= 0 &&
      props.global.userInfo
    if(needMark()) {
      waterMark({
        watermark_txt: `${props.global.userInfo.fullName}（${props.global.userInfo.loginName}）`
      });
    }
    return () => {
      const marks = document.querySelectorAll('.pandawatermark_mask_div')
      if(marks && marks.length) {
        Array.prototype.forEach.call(marks, function(dom, index) {
          document.body.removeChild(dom)
        })
      }
    }
  }, [props.global.userInfo]);


  const onMenuHeaderClick = event => {
    // 非云平台不做响应
    if(!props.global.get('userInfo.site')) return;

    // 至少2个行业才能回到引导页
    const Industries = props.global.get('userInfo.Industries');
    if(Industries && Industries.length > 1) {
      history.push(`/industry`)
    }
  };
  // useEffect(async () => {
  //   setMenuLoading(false);
  // }, [props.menu]);

  const onTopMenuHeaderClick = index => {
    props.updateCurrentIndex(index);
    window.share && window.share.event.emit('trigger:updateMenuIndex', index);
    store.set('updateMenuIndex', index);
  };

  const logo =
    props.global && _.isFunction(props.global.get) && props.global.get('bannerLogo')
      ? props.global &&
        _.isFunction(props.global.transformDevAssetsBaseURL) &&
        props.global.transformDevAssetsBaseURL(props.global.get('bannerLogo'))
      : defaultSetting.logo;

  return (
    <SecurityLayout {...props}>
      <BasicLayout
        title={props.global.title}
        route={props.route}
        location={location}
        navTheme={props.global?.variableTheme?.navTheme ?? 'dark'}
        mode="MDI"
        fixedHeader
        headerHeight={52}
        routeConfig={routeConfig}
        matchPath={matchPath}
        actionRef={actionRef}
        menu={{
          loading: menuLoading
          // onLoadingChange: (loading) => setMenuLoading(loading)
        }}
        logo={logo}
        topMenuActiveKey={props.currentMenuIndex}
        rightContentRender={() => <RightContent/>}
        onPageChange={() => {
          // if(!props.global.token) {
          //   history.push('/user/login')
          // }
        }}
        onMenuHeaderClick={onMenuHeaderClick}
        onTopMenuHeaderClick={onTopMenuHeaderClick}
        menuProps = {{
          popupClassName: 'testpop',
          popupOffset: [0, 15]
        }}
        headerSiteRender={() =>
          renderSite({
            data: cityData,
            config: props.global,
            loading: siteLoading,
            setLoading: setSiteLoading,
            action: siteAction,
            actionRef,
            setMenuLoading,
          })
        }
        menuItemRender={(item, dom) => (

          <a
            onClick={(event) => {
              event.preventDefault()
              window.history.pushState(null, '', '/civbase' + item.path);
            }}
          >
            {dom}
          </a>
        )}
      >

        <Suspense fallback={<></>}>
          {/* {window.location.pathname.startsWith('/civbase/civ_3d') ? ( */}
          {/*  <KeepAlive> */}
          {/*    <CesiumMap /> */}
          {/*  </KeepAlive> */}
          {/* ) : window.location.pathname.startsWith('/civbase/civ_webgis') ? ( */}
          {/*  mapMode === 'AMap' ? ( */}
          {/*    <AMap /> */}
          {/*  ) : ( */}
          {/*    <ArcgisMap options={{ type: 'ArcgisMap' }} /> */}
          {/*  ) */}
          {/* ) : null} */}
          {window.location.pathname.startsWith('/civbase/civ_3d/map') ? (
            <KeepAlive>
              <CesiumMap />
            </KeepAlive>
          ) : (
            window.location.pathname.startsWith('/civbase/civ_webgis') && null
          )}
        </Suspense>
         <div id="micro-container" className="subapp-container">
            {props.children}

          </div>
      </BasicLayout>
    </SecurityLayout>
  );
};

const mapStateToProps = state => ({
  global: state.getIn(['global', 'globalConfig']),
  menu: state.getIn(['global', 'menu']),
  flatMenu: state.getIn(['global', 'flatMenu']),
  currentMenuIndex: state.getIn(['global', 'currentMenuIndex']),
});
const mapDispatchToProps = dispatch => ({
  updageSetting(setting) {
    dispatch(actionCreators.updageSetting(setting));
  },
  updateConfig(config) {
    dispatch(actionCreators.getConfig(config));
  },
  updateCurrentIndex(index) {
    dispatch(actionCreators.updateCurrentIndex(index));
  },
  logout() {
    dispatch(actionCreators.logout());
  },
});
export default connect(
  mapStateToProps,
  mapDispatchToProps,
)(Layout);
